/**
 *  \file IMP/multifit.h
 *  \brief Include all non-deprecated headers in IMP.multifit.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_MULTIFIT_H
#define IMP_MULTIFIT_H
#include <IMP/multifit/AlignmentParams.h>
#include <IMP/multifit/ComplementarityRestraint.h>
#include <IMP/multifit/DataPointsAssignment.h>
#include <IMP/multifit/DensityDataPoints.h>
#include <IMP/multifit/FittingSolutionRecord.h>
#include <IMP/multifit/GeometricHash.h>
#include <IMP/multifit/RadiusOfGyrationRestraint.h>
#include <IMP/multifit/RigidLeavesRefiner.h>
#include <IMP/multifit/SettingsData.h>
#include <IMP/multifit/WeightedExcludedVolumeRestraint.h>
#include <IMP/multifit/anchor_graph.h>
#include <IMP/multifit/anchor_utilities.h>
#include <IMP/multifit/anchors_reader.h>
#include <IMP/multifit/coarse_molecule.h>
#include <IMP/multifit/connolly_surface.h>
#include <IMP/multifit/density_analysis.h>
#include <IMP/multifit/ensemble_analysis.h>
#include <IMP/multifit/fft_based_rigid_fitting.h>
#include <IMP/multifit/fitting_solutions_reader_writer.h>
#include <IMP/multifit/fitting_states.h>
#include <IMP/multifit/fitting_tools.h>
#include <IMP/multifit/fitting_utils.h>
#include <IMP/multifit/merge_tree_utils.h>
#include <IMP/multifit/path_reader_writer.h>
#include <IMP/multifit/pca_based_rigid_fitting.h>
#include <IMP/multifit/protein_anchors_mapping_reader.h>
#include <IMP/multifit/proteomics_em_alignment_atomic.h>
#include <IMP/multifit/proteomics_reader.h>
#include <IMP/multifit/weighted_excluded_volume.h>
#endif /* IMP_MULTIFIT_H */
