/**
 *  \file IMP/multi_state.h
 *  \brief Include all non-deprecated headers in IMP.multi_state.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_MULTI_STATE_H
#define IMP_MULTI_STATE_H
#include <IMP/multi_state/EnsembleGenerator.h>
#include <IMP/multi_state/MultiStateModel.h>
#include <IMP/multi_state/MultiStateModelScore.h>
#include <IMP/multi_state/SAXSMultiStateModelScore.h>
#include <IMP/multi_state/stat_helpers.h>
#endif /* IMP_MULTI_STATE_H */
