# All scripts that can be called from the imp_sampcon application
_all_commands = ["select_good", "show_stat", "plot_score", "exhaust"]


__version__ = "2.15.0"

def get_module_version():
    '''Return the version of this module, as a string'''
    return "2.15.0"

def get_module_name():
    '''Return the fully-qualified name of this module'''
    return "IMP::sampcon"

def get_data_path(fname):
    '''Return the full path to one of this module's data files'''
    import IMP
    return IMP._get_module_data_path("sampcon", fname)

def get_example_path(fname):
    '''Return the full path to one of this module's example files'''
    import IMP
    return IMP._get_module_example_path("sampcon", fname)
