/**
 *  \file IMP/npc.h
 *  \brief Include all non-deprecated headers in IMP.npc.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_NPC_H
#define IMP_NPC_H
#include <IMP/npc/CompositeRestraint.h>
#include <IMP/npc/MinimumSphereDistancePairScore.h>
#include <IMP/npc/ProteinLocalizationRestraint.h>
#endif /* IMP_NPC_H */
