/**
 *  \file IMP/example.h
 *  \brief Include all non-deprecated headers in IMP.example.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_EXAMPLE_H
#define IMP_EXAMPLE_H
#include <IMP/example/ExampleComplexRestraint.h>
#include <IMP/example/ExampleConstraint.h>
#include <IMP/example/ExampleDecorator.h>
#include <IMP/example/ExampleObject.h>
#include <IMP/example/ExamplePairScore.h>
#include <IMP/example/ExampleRestraint.h>
#include <IMP/example/ExampleSingletonModifier.h>
#include <IMP/example/ExampleSubsetFilterTable.h>
#include <IMP/example/ExampleTemplateClass.h>
#include <IMP/example/ExampleUnaryFunction.h>
#include <IMP/example/complex_assembly.h>
#include <IMP/example/counting.h>
#include <IMP/example/creating_restraints.h>
#include <IMP/example/optimizing.h>
#include <IMP/example/randomizing.h>
#endif /* IMP_EXAMPLE_H */
