/**
 *  \file npctransport/npctransport_proto.fwd.h
 *  \brief forward declaration of incomplete protobuf files
 *         for the main data structures used
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMPNPCTRANSPORT_NPCTRANSPORT_PROTO_FWD_H
#define IMPNPCTRANSPORT_NPCTRANSPORT_PROTO_FWD_H


#include "npctransport_config.h"

namespace npctransport_proto {
  class Assignment;
  class Configuration;
  class Statistics;
  class Assignment_FGAssignment;
  class Assignment_InteractionAssignment;
  class Assignment_FloaterAssignment;
  class Assignment_ObstacleAssignment;
}

#endif /* IMPNPCTRANSPORT_NPCTRANSPORT_PROTO_FWD_H */
