// Autogenerated by tools\build\make_containers.py
// from tools\build\container_templates\container\ClassnamesOptimizerState.h
// Do not edit - any changes will be lost!

/**
 *  \file IMP/container/QuadsOptimizerState.h
 *  \brief Use a QuadModifier applied to a ParticleQuadsTemp to
 *  maintain an invariant
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMPCONTAINER_QUADS_OPTIMIZER_STATE_H
#define IMPCONTAINER_QUADS_OPTIMIZER_STATE_H

#include <IMP/container/container_config.h>
#include <IMP/QuadContainer.h>
#include <IMP/QuadModifier.h>
#include <IMP/OptimizerState.h>
#include <IMP/Pointer.h>

IMPKERNEL_BEGIN_NAMESPACE
// for swig
class QuadContainer;
class QuadModifier;
IMPKERNEL_END_NAMESPACE

IMPCONTAINER_BEGIN_NAMESPACE
//! Apply a QuadFunction to a QuadContainer to maintain an invariant
/** \ingroup restraint
    \see QuadOptimizerState
 */
class IMPCONTAINEREXPORT QuadsOptimizerState : public OptimizerState {
  IMP::PointerMember<QuadModifier> f_;
  IMP::PointerMember<QuadContainer> c_;

 public:
  /** \param[in] c The Container to hold the elements to process
      \param[in] gf The QuadModifier to apply to all elements.
      \param[in] name The name to use for this Object
   */
  QuadsOptimizerState(QuadContainerAdaptor c, QuadModifier *gf,
                           std::string name = "QuadsOptimizerState %1%");

  virtual void update() IMP_OVERRIDE;
  IMP_OBJECT_METHODS(QuadsOptimizerState);
};

IMP_OBJECTS(QuadsOptimizerState, QuadsOptimizerStates);

IMPCONTAINER_END_NAMESPACE

#endif /* IMPCONTAINER_QUADS_OPTIMIZER_STATE_H */
