/**
 *  \file IMP/em/def.h
 *  \brief Definitions for EMBED.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPEM_DEF_H
#define IMPEM_DEF_H

#include <IMP/em/em_config.h>

IMPEM_BEGIN_NAMESPACE

typedef double emreal;
const double EPS = 1e-16;

IMPEM_END_NAMESPACE

#endif /* IMPEM_DEF_H */
