/**
 *  \file IMP/saxs.h
 *  \brief Include all non-deprecated headers in IMP.saxs.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_SAXS_H
#define IMP_SAXS_H
#include <IMP/saxs/ChiFreeScore.h>
#include <IMP/saxs/ChiScore.h>
#include <IMP/saxs/ChiScoreLog.h>
#include <IMP/saxs/DerivativeCalculator.h>
#include <IMP/saxs/Distribution.h>
#include <IMP/saxs/FitParameters.h>
#include <IMP/saxs/FormFactorTable.h>
#include <IMP/saxs/Profile.h>
#include <IMP/saxs/ProfileClustering.h>
#include <IMP/saxs/ProfileFitter.h>
#include <IMP/saxs/RadiusOfGyrationRestraint.h>
#include <IMP/saxs/RatioVolatilityScore.h>
#include <IMP/saxs/Restraint.h>
#include <IMP/saxs/RigidBodiesProfileHandler.h>
#include <IMP/saxs/SolventAccessibleSurface.h>
#include <IMP/saxs/WeightedFitParameters.h>
#include <IMP/saxs/WeightedProfileFitter.h>
#include <IMP/saxs/nnls.h>
#include <IMP/saxs/utility.h>
#endif /* IMP_SAXS_H */
