// Autogenerated by tools\build\make_containers.py
// from tools\build\container_templates\kernel\ClassnameDerivativeModifier.h
// Do not edit - any changes will be lost!

/**
 *  \file IMP/QuadDerivativeModifier.h
 *  \brief A Modifier on ParticleQuadsTemp
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMPKERNEL_QUAD_DERIVATIVE_MODIFIER_H
#define IMPKERNEL_QUAD_DERIVATIVE_MODIFIER_H

#include <IMP/kernel_config.h>
#include "DerivativeAccumulator.h"
#include "base_types.h"
#include "internal/container_helpers.h"
#include "QuadModifier.h"

IMPKERNEL_BEGIN_NAMESPACE
typedef QuadModifier QuadDerivativeModifier;

IMPKERNEL_END_NAMESPACE

#endif /* IMPKERNEL_QUAD_DERIVATIVE_MODIFIER_H */
