/**
 *  \file IMP/domino.h
 *  \brief Include all non-deprecated headers in IMP.domino.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_DOMINO_H
#define IMP_DOMINO_H
#include <IMP/domino/Assignment.h>
#include <IMP/domino/BranchAndBoundSampler.h>
#include <IMP/domino/DependencyScoreState.h>
#include <IMP/domino/DiscreteSampler.h>
#include <IMP/domino/DominoSampler.h>
#include <IMP/domino/Order.h>
#include <IMP/domino/Slice.h>
#include <IMP/domino/Subset.h>
#include <IMP/domino/analysis.h>
#include <IMP/domino/assignment_containers.h>
#include <IMP/domino/assignment_tables.h>
#include <IMP/domino/domino_macros.h>
#include <IMP/domino/interactive.h>
#include <IMP/domino/particle_states.h>
#include <IMP/domino/subset_filters.h>
#include <IMP/domino/subset_graphs.h>
#include <IMP/domino/subset_scores.h>
#include <IMP/domino/utility.h>
#endif /* IMP_DOMINO_H */
