/**
 *  \file IMP/modeller.h
 *  \brief Include all non-deprecated headers in IMP.modeller.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_MODELLER_H
#define IMP_MODELLER_H
#endif /* IMP_MODELLER_H */
