/**
 *  \file IMP/base_macros.h
 *  \brief Various general useful macros for IMP.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPKERNEL_BASE_MACROS_H
#define IMPKERNEL_BASE_MACROS_H
// This must not include anything else
#include "internal/eat.h"

#include "compiler_macros.h"
#include "hash_macros.h"
#include "showable_macros.h"
#include "bracket_macros.h"
#include "swap_macros.h"
#include "object_macros.h"
#include "tuple_macros.h"
#include "comparison_macros.h"
#include "raii_macros.h"
#include "value_macros.h"
#include "graph_macros.h"
#include "ref_counted_macros.h"
#include "warning_macros.h"
#include "doxygen_macros.h"
#include "swig_macros.h"
#include "utility_macros.h"

#endif /* IMPKERNEL_BASE_MACROS_H */
