# Autogenerated by tools\build\make_module_version.py
# Do not edit - any changes will be lost!

def check_version(myversion):
  def _check_one(name, expected, found):
      if expected != found:
          raise RuntimeError(
              "Expected version " + expected + " but got " + found
              + " when loading module " + name
              + ". Please make sure IMP is properly built and installed "
              + "and that matching python and C++ libraries are used.")
  version = '2.15.0'
  _check_one('example', version, myversion)
  