/**
 *  \file IMP/kmeans.h
 *  \brief Include all non-deprecated headers in IMP.kmeans.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_KMEANS_H
#define IMP_KMEANS_H
#include <IMP/kmeans/KMeans.h>
#endif /* IMP_KMEANS_H */
