/**
 *  \file internal/utility.h
 *  \brief Various useful utilities
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMPKERNEL_INTERNAL_STATIC_H
#define IMPKERNEL_INTERNAL_STATIC_H

#include <IMP/kernel_config.h>
#include <boost/unordered_map.hpp>

IMPKERNEL_BEGIN_INTERNAL_NAMESPACE

IMPKERNEL_END_INTERNAL_NAMESPACE

#endif /* IMPKERNEL_INTERNAL_STATIC_H */
