/**
 *  \file IMP/container.h
 *  \brief Include all non-deprecated headers in IMP.container.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_CONTAINER_H
#define IMP_CONTAINER_H
#include <IMP/container/AllBipartitePairContainer.h>
#include <IMP/container/AllPairContainer.h>
#include <IMP/container/CloseBipartitePairContainer.h>
#include <IMP/container/ClosePairContainer.h>
#include <IMP/container/ConnectingPairContainer.h>
#include <IMP/container/ConsecutivePairContainer.h>
#include <IMP/container/generic.h>
#include <IMP/container/SingletonContainerSet.h>
#include <IMP/container/SingletonContainerStatistics.h>
#include <IMP/container/SingletonsConstraint.h>
#include <IMP/container/SingletonsOptimizerState.h>
#include <IMP/container/SingletonsRestraint.h>
#include <IMP/container/DistributeSingletonsScoreState.h>
#include <IMP/container/DynamicListSingletonContainer.h>
#include <IMP/container/EventSingletonsOptimizerState.h>
#include <IMP/container/InContainerSingletonFilter.h>
#include <IMP/container/ListSingletonContainer.h>
#include <IMP/container/MinimumSingletonRestraint.h>
#include <IMP/container/MinimumSingletonScore.h>
#include <IMP/container/PredicateSingletonsRestraint.h>
#include <IMP/container/PairContainerSet.h>
#include <IMP/container/PairContainerStatistics.h>
#include <IMP/container/PairsConstraint.h>
#include <IMP/container/PairsOptimizerState.h>
#include <IMP/container/PairsRestraint.h>
#include <IMP/container/DistributePairsScoreState.h>
#include <IMP/container/DynamicListPairContainer.h>
#include <IMP/container/EventPairsOptimizerState.h>
#include <IMP/container/InContainerPairFilter.h>
#include <IMP/container/ListPairContainer.h>
#include <IMP/container/MinimumPairRestraint.h>
#include <IMP/container/MinimumPairScore.h>
#include <IMP/container/PredicatePairsRestraint.h>
#include <IMP/container/TripletContainerSet.h>
#include <IMP/container/TripletContainerStatistics.h>
#include <IMP/container/TripletsConstraint.h>
#include <IMP/container/TripletsOptimizerState.h>
#include <IMP/container/TripletsRestraint.h>
#include <IMP/container/DistributeTripletsScoreState.h>
#include <IMP/container/DynamicListTripletContainer.h>
#include <IMP/container/EventTripletsOptimizerState.h>
#include <IMP/container/InContainerTripletFilter.h>
#include <IMP/container/ListTripletContainer.h>
#include <IMP/container/MinimumTripletRestraint.h>
#include <IMP/container/MinimumTripletScore.h>
#include <IMP/container/PredicateTripletsRestraint.h>
#include <IMP/container/QuadContainerSet.h>
#include <IMP/container/QuadContainerStatistics.h>
#include <IMP/container/QuadsConstraint.h>
#include <IMP/container/QuadsOptimizerState.h>
#include <IMP/container/QuadsRestraint.h>
#include <IMP/container/DistributeQuadsScoreState.h>
#include <IMP/container/DynamicListQuadContainer.h>
#include <IMP/container/EventQuadsOptimizerState.h>
#include <IMP/container/InContainerQuadFilter.h>
#include <IMP/container/ListQuadContainer.h>
#include <IMP/container/MinimumQuadRestraint.h>
#include <IMP/container/MinimumQuadScore.h>
#include <IMP/container/PredicateQuadsRestraint.h>
#endif /* IMP_CONTAINER_H */
