/**
 *  \file IMP/gsl.h
 *  \brief Include all non-deprecated headers in IMP.gsl.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_GSL_H
#define IMP_GSL_H
#include <IMP/gsl/ConjugateGradients.h>
#include <IMP/gsl/GSLOptimizer.h>
#include <IMP/gsl/QuasiNewton.h>
#include <IMP/gsl/Simplex.h>
#endif /* IMP_GSL_H */
