/**
 *  \file IMP/macros.h    \brief Various general useful macros for IMP.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPKERNEL_MACROS_H
#define IMPKERNEL_MACROS_H
#include <IMP/kernel_config.h>
#include <IMP/base_macros.h>
#include <IMP/decorator_macros.h>
#include <IMP/singleton_macros.h>
#include <IMP/pair_macros.h>
#include <IMP/triplet_macros.h>
#include <IMP/quad_macros.h>

#endif /* IMPKERNEL_MACROS_H */
