/**
 *  \file IMP/utility.h
 *  \brief For backwards compatibility.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPKERNEL_UTILITY_H
#define IMPKERNEL_UTILITY_H

#include <IMP/kernel_config.h>
#include <IMP/base_utility.h>

IMPKERNEL_BEGIN_NAMESPACE
IMPKERNEL_END_NAMESPACE

#endif /* IMPKERNEL_UTILITY_H */
