/**
 *  \file IMP/em.h
 *  \brief Include all non-deprecated headers in IMP.em.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_EM_H
#define IMP_EM_H
#include <IMP/em/BayesEM3D.h>
#include <IMP/em/CoarseCC.h>
#include <IMP/em/CoarseCCatIntervals.h>
#include <IMP/em/CoarseConvolution.h>
#include <IMP/em/DensityFillingRestraint.h>
#include <IMP/em/DensityHeader.h>
#include <IMP/em/DensityMap.h>
#include <IMP/em/EMReaderWriter.h>
#include <IMP/em/EnvelopeFitRestraint.h>
#include <IMP/em/EnvelopePenetrationRestraint.h>
#include <IMP/em/EnvelopeScore.h>
#include <IMP/em/FitRestraint.h>
#include <IMP/em/FitRestraintBayesEM3D.h>
#include <IMP/em/ImageHeader.h>
#include <IMP/em/KernelParameters.h>
#include <IMP/em/MRCReaderWriter.h>
#include <IMP/em/MapDistanceTransform.h>
#include <IMP/em/MapReaderWriter.h>
#include <IMP/em/PCAAligner.h>
#include <IMP/em/PCAFitRestraint.h>
#include <IMP/em/SampledDensityMap.h>
#include <IMP/em/SpiderHeader.h>
#include <IMP/em/SpiderReaderWriter.h>
#include <IMP/em/SurfaceShellDensityMap.h>
#include <IMP/em/Voxel.h>
#include <IMP/em/XplorReaderWriter.h>
#include <IMP/em/converters.h>
#include <IMP/em/def.h>
#include <IMP/em/density_map_volumetrics.h>
#include <IMP/em/density_utilities.h>
#include <IMP/em/embedding.h>
#include <IMP/em/envelope_penetration.h>
#include <IMP/em/exp.h>
#include <IMP/em/header_converters.h>
#include <IMP/em/masking.h>
#include <IMP/em/rigid_fitting.h>
#endif /* IMP_EM_H */
