/**
 *  \file IMP/mpi.h
 *  \brief Include all non-deprecated headers in IMP.mpi.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_MPI_H
#define IMP_MPI_H
#include <IMP/mpi/ReplicaExchange.h>
#endif /* IMP_MPI_H */
