/**
 *  \file IMP/statistics.h
 *  \brief Include all non-deprecated headers in IMP.statistics.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_STATISTICS_H
#define IMP_STATISTICS_H
#include <IMP/statistics/ChiSquareMetric.h>
#include <IMP/statistics/Embedding.h>
#include <IMP/statistics/HistogramD.h>
#include <IMP/statistics/Metric.h>
#include <IMP/statistics/PartitionalClustering.h>
#include <IMP/statistics/PartitionalClusteringWithCenter.h>
#include <IMP/statistics/RecursivePartitionalClusteringEmbedding.h>
#include <IMP/statistics/RecursivePartitionalClusteringMetric.h>
#include <IMP/statistics/embeddings.h>
#include <IMP/statistics/metric_clustering.h>
#include <IMP/statistics/metrics.h>
#include <IMP/statistics/point_clustering.h>
#include <IMP/statistics/python_only.h>
#include <IMP/statistics/utility.h>
#endif /* IMP_STATISTICS_H */
