/**
 *  \file IMP/rotamer.h
 *  \brief Include all non-deprecated headers in IMP.rotamer.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_ROTAMER_H
#define IMP_ROTAMER_H
#include <IMP/rotamer/RotamerCalculator.h>
#include <IMP/rotamer/RotamerLibrary.h>
#endif /* IMP_ROTAMER_H */
