/**
 *  \file IMP/rmf.h
 *  \brief Include all non-deprecated headers in IMP.rmf.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_RMF_H
#define IMP_RMF_H
#include <IMP/rmf/HierarchyLoadLink.h>
#include <IMP/rmf/HierarchySaveLink.h>
#include <IMP/rmf/RMFWriter.h>
#include <IMP/rmf/SaveOptimizerState.h>
#include <IMP/rmf/associations.h>
#include <IMP/rmf/atom_io.h>
#include <IMP/rmf/atom_links.h>
#include <IMP/rmf/frames.h>
#include <IMP/rmf/geometry_io.h>
#include <IMP/rmf/link_macros.h>
#include <IMP/rmf/links.h>
#include <IMP/rmf/particle_io.h>
#include <IMP/rmf/restraint_io.h>
#include <IMP/rmf/simple_links.h>
#endif /* IMP_RMF_H */
