/**
 *  \file IMP/score_functor.h
 *  \brief Include all non-deprecated headers in IMP.score_functor.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_SCORE_FUNCTOR_H
#define IMP_SCORE_FUNCTOR_H
#include <IMP/score_functor/AddScores.h>
#include <IMP/score_functor/DistancePairScore.h>
#include <IMP/score_functor/Dope.h>
#include <IMP/score_functor/Harmonic.h>
#include <IMP/score_functor/HarmonicLowerBound.h>
#include <IMP/score_functor/HarmonicUpperBound.h>
#include <IMP/score_functor/LinearLowerBound.h>
#include <IMP/score_functor/LoopStatistical.h>
#include <IMP/score_functor/OpenCubicSpline.h>
#include <IMP/score_functor/OrientedSoap.h>
#include <IMP/score_functor/PointToSphereDistance.h>
#include <IMP/score_functor/Score.h>
#include <IMP/score_functor/ScoreUnaryFunction.h>
#include <IMP/score_functor/Shift.h>
#include <IMP/score_functor/SingletonStatistical.h>
#include <IMP/score_functor/Soap.h>
#include <IMP/score_functor/SphereDistance.h>
#include <IMP/score_functor/Statistical.h>
#include <IMP/score_functor/SurfaceDistancePairScore.h>
#include <IMP/score_functor/UnaryFunctionEvaluate.h>
#include <IMP/score_functor/WeightScore.h>
#include <IMP/score_functor/distance_pair_score_macros.h>
#include <IMP/score_functor/surface_distance_pair_score_macros.h>
#endif /* IMP_SCORE_FUNCTOR_H */
