/**
 *  \file IMP/rmf/atom_links.h
 *  \brief Manage links between an RMF file and an atom::Hierarchy.
 *         This file is only a wrapper for HierarchyLoadLink and
 *         HierarchySaveLink - the name is misleading and may be
 *         changed to atom_hierarchy_links.h in future.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPRMF_ATOM_LINKS_H
#define IMPRMF_ATOM_LINKS_H

#include "HierarchyLoadLink.h"
#include "HierarchySaveLink.h"

#endif /* IMPRMF_ATOM_LINKS_H */
