/**
 *  \file IMP/cgal.h
 *  \brief Include all non-deprecated headers in IMP.cgal.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_CGAL_H
#define IMP_CGAL_H
#endif /* IMP_CGAL_H */
