/**
 *  \file IMP/bayesianem.h
 *  \brief Include all non-deprecated headers in IMP.bayesianem.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_BAYESIANEM_H
#define IMP_BAYESIANEM_H
#include <IMP/bayesianem/GaussianEMRestraint.h>
#include <IMP/bayesianem/utilities.h>
#endif /* IMP_BAYESIANEM_H */
