/**
 *  \file IMP/atom/atom_macros.h    \brief Various important macros
 *                           for implementing decorators.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPATOM_MACROS_H
#define IMPATOM_MACROS_H

#define IMP_ATOM_TYPE_INDEX 8974343
#define IMP_RESIDUE_TYPE_INDEX 90784334
#define IMP_HIERARCHY_TYPE_INDEX 90784335
#define IMP_CHAIN_TYPE_INDEX 90784336

#endif /* IMPATOM_MACROS_H */
