/**
 *  \file IMP/display/geometry.h
 *  \brief Backwards compatibility header.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMPDISPLAY_GEOMETRY_H
#define IMPDISPLAY_GEOMETRY_H

#include <IMP/display/display_config.h>
#include "declare_Geometry.h"
#include "primitive_geometries.h"

IMPDISPLAY_BEGIN_NAMESPACE

IMPDISPLAY_END_NAMESPACE

#endif /* IMPDISPLAY_GEOMETRY_H */
