/**
 *  \file IMP/algebra/Vector2D.h   \brief Simple 2D vector class.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPALGEBRA_VECTOR_2D_H
#define IMPALGEBRA_VECTOR_2D_H

#include <IMP/types.h>
#include <IMP/base_macros.h>
#include <IMP/exception.h>

#include <cmath>

#include "VectorD.h"

IMPALGEBRA_BEGIN_NAMESPACE

IMPALGEBRA_END_NAMESPACE

#endif /* IMPALGEBRA_VECTOR_2D_H */
