#ifndef IMPNPCTRANSPORT_AVRO_H
#define IMPNPCTRANSPORT_AVRO_H
#include <IMP/npctransport/npctransport_config.h>
#include <ValidSchema.hh>

IMPNPCTRANSPORT_BEGIN_NAMESPACE

IMPNPCTRANSPORTEXPORT IMP_NPCTRANSPORT_AVRO_NAMESPACE::ValidSchema get_avro_data_file_schema();

IMPNPCTRANSPORT_END_NAMESPACE

#endif /* IMPNPCTRANSPORT_AVRO_H */
