/**
 *  \file IMP/benchmark.h
 *  \brief Include all non-deprecated headers in IMP.benchmark.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMP_BENCHMARK_H
#define IMP_BENCHMARK_H
#include <IMP/benchmark/Profiler.h>
#include <IMP/benchmark/benchmark_macros.h>
#include <IMP/benchmark/utility.h>
#endif /* IMP_BENCHMARK_H */
