/**
 *  \file IMP/display/display_macros.h
 *  \brief macros for display classes
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 */

#ifndef IMPDISPLAY_MACROS_H
#define IMPDISPLAY_MACROS_H

#include "writer_macros.h"
#include "geometry_macros.h"

#endif /* IMPDISPLAY_MACROS_H */
