



foxs --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
spb_test_score --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
imp_sampcon --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
multifit --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
