/**
 *  \file IMP/core/PeriodicOptimizerState.h
 *  \brief Angle restraint between three particles.
 *
 *  Copyright 2007-2021 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPCORE_PERIODIC_OPTIMIZER_STATE_H
#define IMPCORE_PERIODIC_OPTIMIZER_STATE_H

#include <IMP/core/core_config.h>
#include <IMP/OptimizerState.h>

IMPCORE_BEGIN_NAMESPACE

typedef OptimizerState PeriodicOptimizerState;

IMPCORE_END_NAMESPACE

#endif /* IMPCORE_PERIODIC_OPTIMIZER_STATE_H */
